package com.company.bw_emerson_fallah.Stellar;

import java.io.IOException;
import org.stellar.sdk.AccountRequiresMemoException;
import org.stellar.sdk.AssetTypeNative;
import org.stellar.sdk.KeyPair;
import org.stellar.sdk.Memo;
import org.stellar.sdk.Network;
import org.stellar.sdk.PaymentOperation;
import org.stellar.sdk.Transaction;
import org.stellar.sdk.responses.AccountResponse;
import org.stellar.sdk.responses.SubmitTransactionResponse;

/**
 *
 * @author emerson
 */
public class TransactionManager {

    private KeyPair destination;
    private KeyPair source;

    /**
     * Transactions constructor with parameters
     *
     * @param sourceKey
     * @param destinationKey
     */
    public TransactionManager(String sourceKey, String destinationKey) {
        this.destination = KeyPair.fromAccountId(destinationKey);
        this.source = KeyPair.fromSecretSeed(sourceKey);
    }

    /**
     * Transactions empty constructor
     */
    public TransactionManager() {
    }

    /**
     * Transaction source getter
     *
     * @return transaction source
     */
    public KeyPair getSource() {
        return source;
    }

    /**
     * Transaction destination getter
     *
     * @return transaction destination
     */
    public KeyPair getDestination() {
        return destination;
    }
    
    
    /**
     * This method checks if the destination destination exists or if it is valid
     *
     * @return
     */
    public boolean doesAccountExist() {
        try {
            StringUtil.SERVER.accounts().account(this.getDestination().getAccountId());
            return true;
        } catch (IOException ex) {
            ex.getMessage();
        }
        return false;
    }

    /**
     * This method prepares / builds up the transaction so it can be sent
     *
     * @param amount
     * @param transactionFee
     * @param source
     * @param network
     * @param memo
     * @return transaction
     */
    private Transaction setUpTransaction(String amount, int transactionFee, AccountResponse source, Network network, String memo) {

        try {

            // Start building the transaction.
            Transaction transaction = new Transaction.Builder(source, network)
                    .addOperation(new PaymentOperation.Builder(getDestination().getAccountId(), new AssetTypeNative(), amount).build())
                    // A memo allows you to add your own metadata to a transaction. It's
                    // optional and does not affect how Stellar treats the transaction.
                    .addMemo(Memo.text(memo))
                    .setTimeout(180)
                    .setBaseFee(transactionFee)
                    .build();
            // Sign the transaction to prove you are actually the person sending it.
            transaction.sign(this.getSource());

            return transaction;

        } catch (Exception e) {
        }
        return null;
    }

    /**
     * This method submits the transaction to the server
     *
     * @param String amount
     */
    private void submitTransaction(String amount) {
        try {
            AccountResponse sourceAccount = StringUtil.SERVER.accounts().account(getSource().getAccountId());
            int transactionFee = Transaction.MIN_BASE_FEE;
            Network network = Network.TESTNET;
            String memoMessage = "Test Memo";

            Transaction transaction = setUpTransaction(amount, transactionFee, sourceAccount, network, memoMessage);

            SubmitTransactionResponse response = StringUtil.SERVER.submitTransaction(transaction);
            System.out.println("Success, the payment went through!");
            System.out.println(response);
        } catch (IOException | AccountRequiresMemoException e) {
            System.out.println("Something went wrong when sending the payment!");
            System.out.println(e.getMessage());
        }
    }

    /**
     * This method is used for sending the payment to the destination destination
     *
     * @param amount
     */
    public void sendTransaction(String amount) {
        boolean confirmDestination = doesAccountExist();

        if (confirmDestination) {
            System.out.println("DESTINATION ACCOUNT EXISTS");
            submitTransaction(amount);
        } else {
            System.out.println("DESTINATION ACCOUNT DOESN'T EXISTS");
        }
    }

    /**
     * This method checks the destination destination balance
     * @throws java.io.IOException
     */
    public void checkDestinationBalance() throws IOException {

        AccountResponse destination = StringUtil.SERVER.accounts().account(this.getDestination().getAccountId());

        System.out.println();
        System.out.println("Destination Account: " + this.getDestination().getAccountId());

        for (AccountResponse.Balance balance : destination.getBalances()) {
            System.out.println(String.format("Balance: %s", balance.getBalance()));
        }
    }

    /**
     * This method checks the source destination balance
     * @throws java.io.IOException
     */
    public void checkSourceBalance() throws IOException {

        AccountResponse source = StringUtil.SERVER.accounts().account(this.getSource().getAccountId());

        System.out.println();
        System.out.println("Source Account: " + this.getSource().getAccountId());

        for (AccountResponse.Balance balance : source.getBalances()) {
            System.out.println(String.format("Balance: %s", balance.getBalance()));
        }
    }

}
